#!/usr/bin/perl
######################################################################
# File: $Id: patch.pl 2014/11/13 17:11:49 (Commit Hash): 0b8066c9e19e2859c88f5f17f5ad326ce1a4b58c Exp $
#
# Description: MG install package generator
#
# First Created on 2005/05/20 by mclee@openfind.com.tw
# Last Modified on $Date$ by $Author$
# 
# Description: MailGates Upgrade Script
# Copyright (C)2007 Openfind Information Technology ,Inc. All rights reserved.

######################################################################
#
# GLOBAL VARIABLE
#
######################################################################
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};
$ENV{'PATH'} = '/bin:/usr/bin';
$DIR_PERM = 0750;
$CP = '/bin/cp';
$MV = '/bin/mv';
$RM = '/bin/rm';
$INSTALL = '/usr/bin/install';
$OWNER = "webmail";
$GROUP = "webmail";
$UID = 0;
$GID = 0;
$WEBMAIL_DIR = '/mailgates';
$TMP_ROOT = "/mailgates/mg/tmp";
$PATCH_ROOT = "$TMP_ROOT/update";
$UPT_DIR = "/mailgates/mg/upt";
$LOG_FILE = "/mailgates/mg/logs/patch.log";
$UPDATE_LIST = "$PATCH_ROOT/update.list";
$REMOVE_LIST = "$PATCH_ROOT/remove.list";
$BACKUP_LIST = "$PATCH_ROOT/backup.list";
$PROCESS_PROGRAM = "$PATCH_ROOT/process.pl";
$MG_BNR = "/mailgates/mg/util/mgbnr.pl";
$MG_SUDO = "/mailgates/mg/util/mg_sudo";

######################################################################
#
# ERROR CODE
#
######################################################################
$ERR_USER_GROUP_NOT_EXIST = 11;
$ERR_BACKUP_LIST_OPEN_FAIL = 12;
$ERR_CANT_OPEN_UPT_DIR = 13;
$ERR_DO_BACKUP_FAIL = 14;
$ERR_CANT_OPEN_NEW_MG_CONF = 15;
$ERR_CANT_OPEN_OLD_MG_CONF = 16;
$ERR_DO_UPDATE_FAIL = 17;

######################################################################
#
# Utility
#
######################################################################
sub exit_code
{
	my $status = shift;

	$exit_value  = $status >> 8;
	$signal_num  = $status & 127;
	$dumped_core = $status & 128;

	return $exit_value;
}

sub perlsu
{
	my ($uid, $cmd) = @_;

	$> = $uid;
	$excode = exit_code($exec_ret = system($cmd));
	logit("PROCESS: perlsu $cmd with $uid! ($excode)");
	$> = 0;
}

sub getDateTime
{
	my $input_sec = shift;
	my ($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdat);
	my $str;

	if (defined($sec)) {
		($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdat) = localtime($input_sec);
	} else {
		($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdat) = localtime(time());
	}
	$str = sprintf("%04d/%02d/%02d %02d:%02d:%02d",
		$year + 1900, $mon + 1, $day, $hour, $min, $sec);

	return $str;
}

sub logit
{
	my $msg = shift;
	my $time_str;
	my $log_str;

	$time_str = &getDateTime();
	$log_str = sprintf("[%s] %s", $time_str, $msg);

	open(LOG, ">>$LOG_FILE");
	flock(LOG, LOCK_EX);
	print LOG $log_str . "\n";
	flock(LOG, LOCK_UN);
	close(LOG);

	chown $UID, $GID, "$LOG_FILE";
}

sub create_dir
{
	my $p = shift;
	my @a = split(/\//, $p);
	my $d = ''; 
	if ($p =~ /^\//){ $d = '/';}
	for ($i=0; $i<$#a; $i++){
		next if ($a[$i] eq '');
		if ($d ne '' && $d ne '/') { $d .= '/'; }
		$d .= $a[$i];
		next if (-d $d);
		if (-e $d){ return undef; }
		if ($d  =~ /^([ &:#-\@\w.]+)$/) {
			$d  = $1; #data is now untainted
		}
		if (!(mkdir $d, $DIR_PERM)){
			return undef;
		}
		chown $UID, $GID, $d;
	}
	return $p;
}

sub DoList
{
	my ($list, $act) = @_;
	my ($line, $cmd);

	if ( !open(FIN, "<$list") ){
		logit("WRANING: $list can't open!");
		return;
	}
	while ( $line = <FIN> ){
		chomp($line);
		if ( $act eq "update" ){
			create_dir($line);
			$cmd = "$CP -rp $PATCH_ROOT$line $line";
			if ($cmd  =~ /^([ &:#-\@\w.]+)$/) {
				$cmd  = $1; #data is now untainted
			}
			$excode = exit_code($exec_ret = system($cmd));
			if ( $excode ){
				$cmd = "$INSTALL -o $UID -g $GID $PATCH_ROOT$line $line";
				if ($cmd  =~ /^([ &:#-\@\w.]+)$/) {
					$cmd  = $1; #data is now untainted
				}
				$excode = exit_code($exec_ret = system($cmd));
			}
			if ( $excode ){
				if (! -e "$PATCH_ROOT$line") {# file does not exist
					logit("PROCESS: $PATCH_ROOT$line does not exist!");
				}else{
					logit("PROCESS: update $line fail($excode)!");
					exit $ERR_DO_UPDATE_FAIL;
				}
			}else{
				logit("PROCESS: update $line success!");
			}
		}elsif ( $act eq "remove" ){
			$cmd = "$RM -rf $line";
			if ($cmd  =~ /^([ &:#-\@\w.]+)$/) {
				$cmd  = $1; #data is now untainted
				$excode = exit_code($exec_ret = system($cmd));
				if ( $excode ){
					logit("PROCESS: remove $line fail($excode)!");
					exit $ERR_DO_UPDATE_FAIL;
				}else{
					logit("PROCESS: remove $line success!");
				}
			} else {
				logit("cmd=$cmd is tainted!");
			}
			
		}
	}
	close(FIN);
}

sub UpdateList
{
	DoList($UPDATE_LIST, "update");
}

sub RemoveList
{
	DoList($REMOVE_LIST, "remove");
}

sub BackupList
{
	# create backup list
	if ( !open(FOUT, ">$BACKUP_LIST") ){
		logit("ERROR: $BACKUP_LIST can't open!");
		exit $ERR_BACKUP_LIST_OPEN_FAIL;
	}
	# add update list
	if ( !open(FIN, "<$UPDATE_LIST") ){
		logit("WRANING: $UPDATE_LIST can't open!");
	}else{
		while ( $line = <FIN> ){
			print FOUT "+$line";
		}
		close(FIN);
	}
	# add remove list
	if ( !open(FIN, "<$REMOVE_LIST") ){
		logit("WRANING: $REMOVE_LIST can't open!");
	}else{
		while ( $line = <FIN> ){
			print FOUT "+$line";
		}
		close(FIN);
	}
	close(FOUT);
	# do backup
	$cmd = "$MG_SUDO $MG_BNR -b -c 'PATCH: $PATCHID' -o category=X -x $BACKUP_LIST";
	$excode = exit_code($exec_ret = system($cmd));
	logit("PROCESS: $cmd! ($excode)");
	if (0 != $excode){
		exit $ERR_DO_BACKUP_FAIL;
	}
}

######################################################################
#
# Start
#
######################################################################
require "$PROCESS_PROGRAM";

logit("PROCESS: PATCH to $PATCHID Start!!!!");

if ((!($UID = getpwnam $OWNER)) || (!($GID = getgrnam $GROUP))) {
	logit("ERROR: User($OWNER)/Group($GROUP) does not exist!\n");
	exit $ERR_USER_GROUP_NOT_EXIST;
}

BeforeProcess();

BackupList();

UpdateList();

RemoveList();

AfterProcess();

logit("PROCESS: PATCH to $PATCHID End!!!!");
print STDOUT "PATCH to $PATCHID End!!!!\n";

